
//-----------------------------------------------------------------------

/*
servo interupts occur when its time to change the state of 1 or more of the
servo pins. This is triggered when timer 0 overflows.
*/
// servo_state         000011111222220000000000000000000001111122222
//
// STEER_PIN           ____-----__________________________-----_____
//
// SENSOR_SERVO_PIN    _________-----__________________________-----
//
//                         \___/\___/\___________________/
//
//                           ^    ^           ^
// steering servo pulse -----'    |           |
// sensor servo pulse ------------'           |
// all servos low (20ms) ---------------------'

#int_TIMER0  //~
void servo_control(){

   // TRANSITION FROM STATE 0 TO STATE 1?
   if(servo_state == 0){
      output_high(STEER_PIN);
      set_timer0(steer_timer_value); // X 12.5ms!
      servo_state++;
   }

   // TRANSITION FROM STATE 1 TO STATE 2?
   else if(servo_state==1){
      output_low(STEER_PIN);
      output_high(SENSOR_SERVO_PIN);
      set_timer0(sensor_servo_timer_value);
      servo_state++;
   }

   // TRANSITION FROM STATE 2 TO STATE 0?
   else{
      output_low(SENSOR_SERVO_PIN);
      set_timer0(servo_low_timer_value); // X 12.5ms!
      servo_state = 0;
   }

}

//----------------------------
// 1 keep track of time in general
// 2 keep the heartbeat LED blinking
// 3 keep track of wheel movement (the odometer)
// 4 keep track of wheel speed

#int_timer2
void millisecond_isr(){

   //  system clock, increment every millisecond
   current_time++;

   // heartbeat
   if(current_time >= blink_time){

      if(hb_state==4){
         output_low(STATUS_LED);
         hb_state=1;
         blink_time = current_time + (int32)hb_on;
      }
      else if(hb_state==1) {
         output_high(STATUS_LED);
         hb_state=2;
         blink_time = current_time + (int32)hb_1;
      }
      else if(hb_state==2){
         output_low(STATUS_LED);
         hb_state=3;
         blink_time = current_time + (int32)hb_on;
      }
      else{
         output_high(STATUS_LED);
         hb_state=4;
         blink_time = current_time + (int32)hb_2;
      }

   }//end of blink if


   // check the front wheel. increment counter if it's moved
   if(wheel_state==0 && input(WHEEL_PIN)==1){
      wheel_state=1;
      wheel_count++;
      wheel_ticks_since_gps_check++;
   }
   else if(wheel_state==1 && input(WHEEL_PIN)==0){
      wheel_state=0;
      wheel_count++;
      wheel_ticks_since_gps_check++;
   }

   // update speed variables as required
   ms_since_update++;
   if(ms_since_update >= update_interval){
      ms_since_update=0;
      wheel_count_old = wheel_count_recent;
      wheel_count_recent = wheel_count;
   }

}//end of int.

